package com.imin.apitest.biz;

import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;
import androidx.core.content.FileProvider;

import android.Manifest;
import android.app.ActivityManager;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageInstaller;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.provider.Settings;
import android.text.TextUtils;
import android.util.Log;
import android.view.Display;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import com.imin.apitest.R;
import com.imin.apitest.utils.DisplayUtils;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.reflect.Method;

/**
 * @author ：iMin Developer
 * @version 1.0
 * @createTime ：2023/10/27 18:18
 * For ALL
 **/
public class DynamicPermissionsActivity extends AppCompatActivity {

    Button shutdown,reboot ,si , ok, displays;
    TextView tvDisplay;
    EditText et;
    ActivityManager am;
    private static final String apkPath = Environment.getExternalStorageDirectory().getPath() + "/test.apk";
    private String apkName ;
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);

        am = (ActivityManager)getSystemService(Context.ACTIVITY_SERVICE);

        shutdown = (Button) this.findViewById(R.id.shutdown);
        reboot =(Button) this.findViewById(R.id.reboot);
        si = (Button) this.findViewById(R.id.silenceInstall);
        et = (EditText)this.findViewById(R.id.tx);
        ok = (Button)this.findViewById(R.id.ok);
        displays = this.findViewById(R.id.btn_display);
        tvDisplay = this.findViewById(R.id.tv_display);

        apkName = getPackageName();
        Log.d("iminSF","current APP package name : " + apkName);

        ok.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if(et.getText() != null && !TextUtils.isEmpty(et.getText())){
                    apkName = String.valueOf(et.getText());
                    Log.d("iminSF","apkName : " + apkName);
                }else{
                    Log.d("iminSF","apkName is null " );
                    apkName = getPackageName();
                }
            }
        });

        shutdown.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Log.d("iminSF"," you clicked shutdown button !!!!!!!!!!!");
                //setShundown();
                setShundownOrReboot(apkName,false);
            }
        });

        reboot.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Log.d("iminSF"," you clicked reboot button !!!!!!!!!!!");
                setShundownOrReboot(apkName,true);
            }
        });

        si.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Log.d("iminSF"," you clicked silence install button !!!!!!!!!!!");
                File file = new File(apkPath);
                if(file.exists()){
                    setSilenceInstall(apkName,apkPath);
                }else {
                    Log.d("iminSF"," apk file  not exists !!!");
                    Toast.makeText(getApplicationContext(),"apk file  not exists  !!!",Toast.LENGTH_LONG).show();
                }
            }
        });

        displays.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Display[] presentationDisplays = DisplayUtils.getPresentationDisplays(DynamicPermissionsActivity.this);
                if(presentationDisplays!=null && presentationDisplays.length >0){
                    StringBuffer sb = new StringBuffer();
                    for (Display display:presentationDisplays){
                        sb.append(display.toString()+"\n");
                    }
                    tvDisplay.setText(sb.toString());
                }else{
                    tvDisplay.setText("获取屏幕方法无效");
                }

            }
        });
        checkPermissions();
    }

    private void checkPermissions() {
        File apkFile = new File(apkPath);
        if (apkFile.canWrite() && apkFile.canRead()) {
            return;
        }
        if (Build.VERSION.SDK_INT >= 33) {
            Intent intent = new Intent(Settings.ACTION_MANAGE_APP_ALL_FILES_ACCESS_PERMISSION)
                    .setData(Uri.parse("package:" + getPackageName()));
            startActivity(intent);
        } else if (Build.VERSION.SDK_INT >= 23) {
            int REQUEST_CODE_CONTACT = 101;
            String[] permissions = {Manifest.permission.WRITE_EXTERNAL_STORAGE,
                    Manifest.permission.MANAGE_EXTERNAL_STORAGE,
                    Manifest.permission.READ_EXTERNAL_STORAGE,
                    Manifest.permission.MOUNT_UNMOUNT_FILESYSTEMS};
            //验证是否许可权限
            for (String str : permissions) {
                if (ActivityCompat.checkSelfPermission(this, str) != PackageManager.PERMISSION_GRANTED) {
                    //申请权限
                    this.requestPermissions(permissions, REQUEST_CODE_CONTACT);
                    return;
                }
            }
        }
    }

    private void setShundownOrReboot(String appName , Boolean isReboot){
        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.TIRAMISU) {
            try {
                Object iMinServiceManager = getSystemService("iminservice");
                Class<?> c = Class.forName("android.app.iMinServiceManager");
                Method method = c.getMethod("setShutdownOrRebootforApp", String.class, boolean.class);
                method.invoke(iMinServiceManager, appName, isReboot);
            } catch (Exception e) {
                e.printStackTrace();
            }
        } else {
            try {
                Class<?> c = Class.forName("android.app.ActivityManager");
                Method method = c.getMethod("setShutdownOrRebootforApp", String.class, boolean.class);
                method.invoke(am, appName, isReboot);
                Log.d("iminSF", "setShundownOrReboot : " + appName);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    private void setSilenceInstall(String apkName , String apkFilePath){
        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.TIRAMISU) {
            try {
//                boolean isInstallPermission = getPackageManager().canRequestPackageInstalls();
//                if(!isInstallPermission){
//                    //权限没有打开则提示用户去手动打开
//                    openInstallPermission();
//                }
                Object iMinServiceManager = getSystemService("iminservice");
                Class<?> c = Class.forName("android.app.iMinServiceManager");
                Method method = c.getMethod("setSilenceInstallforApp",String.class,String.class);
                method.invoke(iMinServiceManager,apkName,apkFilePath);
            } catch (Exception e) {
                e.printStackTrace();
            }
        } else {
            try {
                Class<?> c = Class.forName("android.app.ActivityManager");
                Method method = c.getMethod("setSilenceInstallforApp", String.class, String.class);
                method.invoke(am, apkName, apkFilePath);
                Log.d("iminSF", "setSilenceInstall : " + apkName);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }



    /**
     * 跳转到设置-允许安装未知来源-页面
     */
    private void openInstallPermission() {
        Intent intent = null;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            intent = new Intent(Settings.ACTION_MANAGE_UNKNOWN_APP_SOURCES
                    ,Uri.parse("package:" +getPackageName()));
        }else {
            intent = new Intent(Settings.ACTION_SECURITY_SETTINGS
                    ,Uri.parse("package:" +getPackageName()));

        }

        startActivityForResult(intent, 1);
    }




    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);

        if (resultCode == RESULT_OK && requestCode == 1) {

        }
    }

    private void install2(String path) {
        File apk = new File(path);
        Intent intent = new Intent(Intent.ACTION_VIEW);
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
            //注意第二个参数，要保持和manifest中android:authorities的值相同
            Uri uri = FileProvider.getUriForFile(this,
                    this.getPackageName() + ".fileProvider", apk);
            intent.setDataAndType(uri, "application/vnd.android.package-archive");
        } else {
            intent.setDataAndType(Uri.fromFile(apk), "application/vnd.android.package-archive");
        }
        try {
            startActivity(intent);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private boolean install28(Context context, String apkFilePath) {
        Log.d("iminSF", "install28 path :=" + apkFilePath);
        File apkFile = new File(apkFilePath);
        PackageInstaller packageInstaller = context.getPackageManager().getPackageInstaller();
        PackageInstaller.SessionParams sessionParams
                = new PackageInstaller.SessionParams(PackageInstaller
                .SessionParams.MODE_FULL_INSTALL);
        sessionParams.setSize(apkFile.length());

        int sessionId = createSession(packageInstaller, sessionParams);
        Log.d("iminSF", "install28  sessionId=" + sessionId);
        if (sessionId != -1) {
            boolean copySuccess = copyInstallFile(packageInstaller, sessionId, apkFilePath);
            Log.d("iminSF", "install28  copySuccess=" + copySuccess);
            if (copySuccess) {
                return execInstallCommand(context, packageInstaller, sessionId);
            }
        }
        return false;
    }

    private static int createSession(PackageInstaller packageInstaller,
                                     PackageInstaller.SessionParams sessionParams) {
        int sessionId = -1;
        try {
            sessionId = packageInstaller.createSession(sessionParams);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return sessionId;
    }

    private boolean copyInstallFile(PackageInstaller packageInstaller,
                                           int sessionId, String apkFilePath) {
        InputStream in = null;
        OutputStream out = null;
        PackageInstaller.Session session = null;
        boolean success = false;
        try {
            File apkFile = new File(apkFilePath);
            Log.i("iminSF", "apkFile.canRead()= " + apkFile.canRead() + ", apkFile.canWrite()=" + apkFile.canWrite());
            session = packageInstaller.openSession(sessionId);
            out = session.openWrite("base.apk", 0, apkFile.length());
            Log.i("iminSF", "apkFile.name= " + apkFile.getName() + ", apkFile.length=" + apkFile.length());
            in = new FileInputStream(apkFile);
            int total = 0, c;
            byte[] buffer = new byte[65536];
            while ((c = in.read(buffer)) != -1) {
                total += c;
                out.write(buffer, 0, c);
            }
            session.fsync(out);
            Log.i("iminSF", "streamed " + total + " bytes");
            success = true;
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (out != null) out.close();
                if (in != null) in.close();
                if (session != null) session.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return success;
    }

    private boolean execInstallCommand(Context context, PackageInstaller packageInstaller, int sessionId) {
        PackageInstaller.Session session = null;
        try {
            session = packageInstaller.openSession(sessionId);
            Intent intent = new Intent(context, InstallReceiver.class);

            PendingIntent pendingIntent = PendingIntent.getBroadcast(context,
                    1, intent,
                    PendingIntent.FLAG_IMMUTABLE);
            session.commit(pendingIntent.getIntentSender());
            Log.i("iminSF", "begin session");
            return true;
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (session != null) {
                session.close();
            }
        }
        return false;
    }



    private class InstallReceiver extends BroadcastReceiver {

        @Override
        public void onReceive(Context context, Intent intent) {
            Log.d("iminSF", "收到安装反馈广播了");
            if (intent != null) {
                String packageName = intent.getExtras().getString("pkg");
                String path = intent.getExtras().getString("path");
                final int status = intent.getIntExtra(PackageInstaller.EXTRA_STATUS,
                        PackageInstaller.STATUS_FAILURE);
                if (status != PackageInstaller.STATUS_SUCCESS) {
                    install(context,path);
                }
            }
        }

    }

    public static void install(Context context, String mApkPath) {
        try {
            File uriFile = new File(mApkPath);
            Log.v("iminSF", "uriFile>>" + uriFile);
            try {
                Runtime.getRuntime().exec("chmod 777 " + uriFile);
            } catch (IOException e) {
                e.printStackTrace();
            }
            Intent intent = new Intent(Intent.ACTION_VIEW);
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                intent.setFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION | Intent.FLAG_ACTIVITY_NEW_TASK);
                Uri contentUri = FileProvider.getUriForFile(context, context.getPackageName() + ".fileProvider", uriFile);
                intent.setDataAndType(contentUri, "application/vnd.android.package-archive");
            } else {
                intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                intent.setDataAndType(Uri.fromFile(uriFile), "application/vnd.android.package-archive");
            }
            context.startActivity(intent);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void installApk(String apkPath) {
        boolean hasInstallPermission = getPackageManager().canRequestPackageInstalls();
        if (!hasInstallPermission) {
            startInstallPermissionSettingActivity();
        } else {
            File apk = new File(apkPath);
            Intent intent = new Intent(Intent.ACTION_VIEW);
            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
            Uri uri = FileProvider.getUriForFile(this, "com.imin.apitest.fileprovider", apk);
            intent.setDataAndType(uri, "application/vnd.android.package-archive");
            startActivity(intent);
        }
    }

    /**
     * 申请安装未知应用权限
     */
    private void startInstallPermissionSettingActivity() {
        try {
            Uri packageURI = Uri.parse("package:$packageName");
            //注意这个是8.0新API
            Intent intent = new Intent(Settings.ACTION_MANAGE_UNKNOWN_APP_SOURCES, packageURI);
            startActivityForResult(intent, 1000);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}