package com.imin.apitest.biz;

import android.annotation.SuppressLint;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbManager;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;

import com.imin.apitest.R;
import com.imin.apitest.utils.OnSingleClickListener;
import com.imin.library.IminSDKManager;

import java.util.HashMap;
import java.util.Iterator;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;

/**
 * @author ：iMin Developer
 * @version 1.0
 * @createTime ：2023/11/23 21:01
 * For Crane 1
 **/
public class LightActivity extends AppCompatActivity {
    private static final String TAG = "LightActivity";
    Button btnConnect;
    Button btnTurnOnGreen;
    Button btnTurnOnLed;
    Button btnDisconnect;
    Button btnTurnOff;

    private static final String ACTION_USB_PERMISSION = "android.permission.USB_PERMISSION";
    private static final String ACTION_USB_DEVICE_ATTACHED = "android.hardware.usb.action.USB_DEVICE_ATTACHED";
    private static final String ACTION_USB_DEVICE_DETACHED = "android.hardware.usb.action.USB_DEVICE_DETACHED";
    private UsbManager mUsbManager;
    private UsbDevice usbDevice;

    @SuppressLint("MissingInflatedId")
    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_light);
        btnConnect = findViewById(R.id.bt_connect_light);
        btnTurnOnGreen = findViewById(R.id.bt_turn_on_green);
        btnTurnOnLed = findViewById(R.id.bt_turn_on_led);
        btnDisconnect = findViewById(R.id.bt_disconnect);
        btnTurnOff = findViewById(R.id.bt_turn_off);

        mUsbManager = (UsbManager) this.getSystemService(Context.USB_SERVICE);

        btnConnect.setOnClickListener(new OnSingleClickListener(1000) {
            @Override
            public void onSingleClick(View v) {
                usbDevice = IminSDKManager.getLightDevice(LightActivity.this);
                boolean isConnect = requestPermission(usbDevice);
                handlerOpenCloseLight(isConnect);
            }
        });

        btnTurnOnGreen.setOnClickListener(new OnSingleClickListener(1000) {
            @Override
            public void onSingleClick(View v) {
                IminSDKManager.turnOnGreenLight(LightActivity.this);
            }
        });

        btnTurnOnLed.setOnClickListener(new OnSingleClickListener(1000) {
            @Override
            public void onSingleClick(View v) {
                IminSDKManager.turnOnRedLight(LightActivity.this);
            }
        });

        btnTurnOff.setOnClickListener(new OnSingleClickListener() {
            @Override
            public void onSingleClick(View v) {
                IminSDKManager.turnOffLight(LightActivity.this);
            }
        });


        btnDisconnect.setOnClickListener(new OnSingleClickListener(1000) {
            @Override
            public void onSingleClick(View v) {
                IminSDKManager.disconnectLightDevice(LightActivity.this);
                handlerOpenCloseLight(false);
            }
        });

        handlerOpenCloseLight(false);

    }


    public boolean requestPermission(UsbDevice usbDevice) {
        Log.d(TAG,"requestPermission============");
        if (usbDevice == null){
            return false;
        }

        if (mUsbManager.hasPermission(usbDevice)) {
            boolean isConnect =  IminSDKManager.connectLightDevice(LightActivity.this);
            Log.d(TAG,"usb is connect:"+isConnect);
            return true;
        } else {
            //@SuppressLint("UnspecifiedImmutableFlag") PendingIntent pendingIntent = PendingIntent.getBroadcast(BaseApplication.getAppContext(), 0, new Intent(ACTION_USB_PERMISSION), 0);
            PendingIntent pendingIntent;
            if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.S) {
                pendingIntent = PendingIntent.getBroadcast(this, 0, new Intent(ACTION_USB_PERMISSION), PendingIntent.FLAG_UPDATE_CURRENT | PendingIntent.FLAG_IMMUTABLE);
            } else {
                pendingIntent = PendingIntent.getBroadcast(this, 0, new Intent(ACTION_USB_PERMISSION), 0);
            }

            IntentFilter intentFilter = new IntentFilter(ACTION_USB_PERMISSION);
            intentFilter.addAction(ACTION_USB_DEVICE_ATTACHED);
            intentFilter.addAction(ACTION_USB_DEVICE_DETACHED);
            this.registerReceiver(mUsbDeviceReceiver, intentFilter);
            mUsbManager.requestPermission(usbDevice, pendingIntent);
            return mUsbManager.hasPermission(usbDevice);
        }
    }


    /**
     * 处理开关灯UI逻辑
     * */
    private void handlerOpenCloseLight(boolean isConnect) {
        btnConnect.setEnabled(!isConnect);
        btnTurnOnGreen.setEnabled(isConnect);
        btnTurnOnLed.setEnabled(isConnect);
        btnTurnOff.setEnabled(isConnect);
        btnDisconnect.setEnabled(isConnect);
    }


    public BroadcastReceiver mUsbDeviceReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();
            Log.d(TAG,"UsbDeviceReceiver action = " + action);
            if (ACTION_USB_PERMISSION.equals(action)) {
                android.hardware.usb.UsbDevice device = intent.getParcelableExtra(UsbManager.EXTRA_DEVICE);
                Log.d(TAG,"UsbDeviceReceiver device = " + action);
//                if (intent.getBooleanExtra(UsbManager.EXTRA_PERMISSION_GRANTED, false)) {
//                    if (device != null) {
                        //TODO connectAndOpen
                       boolean isConnect =  IminSDKManager.connectLightDevice(LightActivity.this);
                       handlerOpenCloseLight(isConnect);
                       Log.d(TAG,"是否已连接"+isConnect +"mUsbDeviceReceiver");
//                    }
//                }
            }else if(ACTION_USB_DEVICE_ATTACHED.equals(action)){
                //openUsbDevice();
            }else if(ACTION_USB_DEVICE_DETACHED.equals(action)){
                //TODO closeUsbDevice();
            }
        }
    };
}
