package com.imin.apitest.biz;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;

import com.imin.apitest.R;
import com.imin.apitest.utils.SharedManager;

import java.lang.reflect.Method;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Arrays;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;

/**
 * @author ：iMin Developer
 * @version 1.0
 * @createTime ：2023/10/27 18:18
 * For Crane 1
 **/
public class ScannerActivity extends AppCompatActivity {
    EditText textView;
    EditText etBroadAction;
    EditText etBroadData;
    EditText etBroadByteData;
    Button btnRegister;
    Button btnUnregister;
    Button btnGetStatus;
    public static String DEVICE_CONNECTION = "com.imin.scanner.api.DEVICE_CONNECTION";
    public static String DEVICE_DISCONNECTION = "com.imin.scanner.api.DEVICE_DISCONNECTION";
    public static String RESULT_ACTION = "com.imin.scanner.api.RESULT_ACTION";
    public static String CONNECTION_BACK_ACTION = "com.imin.scanner.api.CONNECTION_RESULT";
    public static String CONNECTION_STATUS_ACTION = "com.imin.scanner.api.DEVICE_IS_CONNECTION";
    public static String LABEL_TYPE = "com.imin.scanner.api.label_type";
    public static String EXTRA_DECODE_DATA = "decode_data"; //extra string
    public static String EXTRA_DECODE_DATA_STR = "decode_data_str"; //extra byte
    public static String CONNECTION_TYPE = "com.imin.scanner.api.status";
    ScannerReceiver scannerReceiver;
    SharedManager sharedManager;
    private static String ET_ACTION = "action";
    private static String ET_BYTE_DATA = "byteData";
    private static String ET_STR_DATA = "strData";
    private int mScanCount = 0;
    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_scanner);
        textView = findViewById(R.id.tv_content);
        etBroadAction = findViewById(R.id.et_broadcast_action);
        etBroadData = findViewById(R.id.et_broadcast_extra_data);
        etBroadByteData = findViewById(R.id.et_broadcast_extra_byte_data);
        btnRegister = findViewById(R.id.bt_register);
        btnUnregister = findViewById(R.id.bt_unregister);
        btnGetStatus = findViewById(R.id.bt_status);
        btnUnregister.setEnabled(false);
        sharedManager = SharedManager.getInstance(this);
        initViewListener();

        textView.setSelection(textView.getText().length());

    }

    private void initViewListener() {
        String action = sharedManager.getString(ET_ACTION);
        String byteData = sharedManager.getString(ET_BYTE_DATA);
        String strData = sharedManager.getString(ET_STR_DATA);

        if(!TextUtils.isEmpty(action)){
            etBroadAction.setText(action);
        }
        if(!TextUtils.isEmpty(byteData)){
            etBroadByteData.setText(byteData);
        }
        if(!TextUtils.isEmpty(strData)){
            etBroadData.setText(strData);
        }

        btnRegister.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                registerScannerBroadcast();
                textView.append("\nregister success");
                setRegisterStatus(true);
            }
        });
        btnUnregister.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                unReisterScannerBroadcast();
                textView.append("\nunregister success");
                setRegisterStatus(false);
            }
        });
        btnGetStatus.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                getScannerStatus();
            }
        });

        //textView.setInputType(InputType);

    }

    private void setRegisterStatus(boolean isRegister) {
        btnRegister.setEnabled(!isRegister);
        btnUnregister.setEnabled(isRegister);
        etBroadAction.setEnabled(!isRegister);
        etBroadByteData.setEnabled(!isRegister);
        etBroadData.setEnabled(!isRegister);
        textView.setEnabled(!isRegister);
        if(isRegister){
            sharedManager.put(ET_ACTION,etBroadAction.getText().toString());
            sharedManager.put(ET_BYTE_DATA,etBroadByteData.getText().toString());
            sharedManager.put(ET_STR_DATA,etBroadData.getText().toString());
        }
    }

    private void unReisterScannerBroadcast() {
        unregisterReceiver(scannerReceiver);
    }

    String mResultAction;
    private void registerScannerBroadcast() {
        mResultAction = etBroadAction.getText().toString();
        if(TextUtils.isEmpty(mResultAction)){
            mResultAction = RESULT_ACTION;
        }
        IntentFilter intentFilter = new IntentFilter();
        intentFilter.addAction(DEVICE_CONNECTION); // scanner connect broadcast receiver action
        intentFilter.addAction(DEVICE_DISCONNECTION); //scanner disconnect broadcast receiver action
        intentFilter.addAction(mResultAction); //scanner content broadcast receiver action
        intentFilter.addAction(CONNECTION_BACK_ACTION); //scanner status
        //create broadcast receiver
        scannerReceiver = new ScannerReceiver();
        //register
        registerReceiver(scannerReceiver, intentFilter);
    }

    DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    class ScannerReceiver extends BroadcastReceiver {
        @Override
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();
            if (action.equals(DEVICE_CONNECTION)) { //scanner is connect
                textView.append("\nUSB Connect");
            } else if (action.equals(DEVICE_DISCONNECTION)) { //scanner is disconnect
                textView.append("\nUSB Disconnect");
            } else if (action.equals(mResultAction)) { //scanner content
                String extraByteData = etBroadByteData.getText().toString();
                String extraData = etBroadData.getText().toString();
                if(TextUtils.isEmpty(extraByteData)){
                    //extraByteData = "source_byte";
                    extraByteData = EXTRA_DECODE_DATA;
                }
                if(TextUtils.isEmpty(extraData)){
                    //extraData = "data";
                    extraData = EXTRA_DECODE_DATA_STR;
                }
                String labelType = intent.getStringExtra(LABEL_TYPE);
                byte[] decodeData = intent.getByteArrayExtra(extraByteData);
                String strData = intent.getStringExtra(extraData);

                textView.append( "\nBroadcast result：byte：" + Arrays.toString(decodeData));
                textView.append( "\nBroadcast result：string：" + strData);
                textView.append( "\nscanner count"+ mScanCount++);
                if(mScanCount % 100 == 0){
                    textView.setText("");
                }
            } else if (action.equals(CONNECTION_BACK_ACTION)) { //获取扫码设备连接状态
                int type = intent.getIntExtra(CONNECTION_TYPE, 0);
                textView.append("\ndevice isConnection is " + (type == 1));
            }

        }
    }

    public static String GET_STATUS = "persist.sys.imin.scanner.status";

    /**
     * getScannerStatus
     */
    public void getScannerStatus() {
        try {
            //type 1:get status by api
            Class<?> c = Class.forName("android.os.SystemProperties");
            Method get = c.getMethod("get", String.class, String.class);
            String connectStatus = (String) get.invoke(c, GET_STATUS, "0");
            if ("1".equals(connectStatus)) {
                Log.d("iMinService", "Connection is " + true);
            }
            //textView.append("\ndevice isConnection is " + ("1".equals(connectStatus)));
            //type 2:get status by broadcast
            Intent intent = new Intent(CONNECTION_STATUS_ACTION);
            sendBroadcast(intent);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void clearText(View view){
        textView.getText().clear();
    }

}
