package com.imin.apitest.biz.display;

import android.app.Presentation;
import android.content.Context;
import android.content.Intent;
import android.hardware.display.DisplayManager;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.provider.Settings;
import android.util.Log;
import android.view.Display;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.MediaController;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.VideoView;

import com.imin.apitest.R;

import androidx.appcompat.app.AppCompatActivity;

/**
 * @author ：Chenjk
 * @version 1.0
 * @createTime ：2023/12/29 10:55
 **/
public class ScreenActivity extends AppCompatActivity {
    private static final String TAG = "display_demo";
    private Presentation presentation;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_screen);
        Button showDisplay = findViewById(R.id.btn_open_display);
        Button showText = findViewById(R.id.btn_show_text);
        Button showPic = findViewById(R.id.btn_show_pic);
        Button showVideo = findViewById(R.id.btn_show_video);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M && Build.VERSION.SDK_INT < Build.VERSION_CODES.S_V2) {
            if (!Settings.canDrawOverlays(this)) {
                Toast.makeText(this, "请同意显示窗口权限", Toast.LENGTH_SHORT).show();
                startActivity(new Intent(Settings.ACTION_MANAGE_OVERLAY_PERMISSION));
            }
        }

        showDisplay.setOnClickListener(v -> {
            if (presentation != null) {
                presentation.cancel();
                presentation = null;
                return;
            }
//                showSecondByMediaRouter(MainActivity.this);
            showSecondByDisplayManager(ScreenActivity.this);

        });
        showText.setOnClickListener(v -> {
            if (presentation != null) {
                //presentation.cancel();
                //presentation..showTextView("");
                TextView textView = presentation.findViewById(R.id.text);
                textView.setText(getString(R.string.dual_screen_text));
            }


        });
        showPic.setOnClickListener(v -> {
            if (presentation != null) {
                ImageView imageView = presentation.findViewById(R.id.image);
                imageView.setVisibility(View.VISIBLE);
                imageView.setImageResource(R.mipmap.imin_product);
            }


        });
        showVideo.setOnClickListener(v -> {
            if (presentation != null) {
                VideoView videoView = presentation.findViewById(R.id.video);
                videoView.setVisibility(View.VISIBLE);
                videoView.setVideoURI(Uri.parse("android.resource://" + getPackageName() + "/" + R.raw.imin_video_3));
                //MediaController controller = new MediaController(ScreenActivity.this);
                //videoView.setMediaController(controller);
                videoView.start();
            }
        });
    }


//    private void showSecondByMediaRouter(Context context){
//        MediaRouter mediaRouter = (MediaRouter) context.getSystemService(Context.MEDIA_ROUTER_SERVICE);
//        MediaRouter.RouteInfo route = mediaRouter.getSelectedRoute(MediaRouter.ROUTE_TYPE_LIVE_VIDEO);
//        if (route != null) {
//             Display presentationDisplay = route.getPresentationDisplay();
//            if (presentationDisplay != null) {
//                presentation = new DifferentDisplay(context, presentationDisplay);
//                presentation.show();
//            }
//        }
//    }

    private void showSecondByDisplayManager(Context context) {
        DisplayManager mDisplayManager = (DisplayManager) getSystemService(Context.DISPLAY_SERVICE);
        Display[] displays = mDisplayManager.getDisplays(DisplayManager.DISPLAY_CATEGORY_PRESENTATION);
        if (displays != null && getPresentationDisplays() != null) {
            presentation = new DifferentDisplay(getApplicationContext(), getPresentationDisplays());
            presentation.show();
        } else {
            Toast.makeText(ScreenActivity.this, getString(R.string.no_second_screen), Toast.LENGTH_SHORT).show();
        }
        /*副屏的Window*/

    }

    public Display getPresentationDisplays() {
        DisplayManager mDisplayManager = (DisplayManager) getSystemService(Context.DISPLAY_SERVICE);
        Display[] displays = mDisplayManager.getDisplays();
        if (displays != null) {
            for (int i = 0; i < displays.length; i++) {
                Log.e(TAG, "屏幕==>" + displays[i] + " Flag:==> " + displays[i].getFlags());
                if ((displays[i].getFlags() & Display.FLAG_SECURE) != 0
                        && (displays[i].getFlags() & Display.FLAG_SUPPORTS_PROTECTED_BUFFERS) != 0
                        && (displays[i].getFlags() & Display.FLAG_PRESENTATION) != 0) {
                    Log.e(TAG, "第一个真实存在的副屏屏幕==> " + displays[i]);
                    return displays[i];
                }
            }
        }

        return null;
    }

//    private void showSecondByActivity(Context context){
//        ActivityOptions options = ActivityOptions.makeBasic();
//        MediaRouter mediaRouter = (MediaRouter) context.getSystemService(Context.MEDIA_ROUTER_SERVICE);
//        MediaRouter.RouteInfo route = mediaRouter.getSelectedRoute(MediaRouter.ROUTE_TYPE_LIVE_VIDEO);
//        if (route != null) {
//            Display presentationDisplay = route.getPresentationDisplay();
//            options.setLaunchDisplayId(presentationDisplay.getDisplayId());
//            Intent intent = new Intent("android.intent.action.MUSIC_PLAYER");
//            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
//            startActivity(intent, options.toBundle());
//        }
//    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (presentation != null) {
            presentation.cancel();
            presentation = null;
        }
    }
}