package com.imin.apitest.biz.scale;


import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import com.imin.apitest.R;
import com.neostra.electronic.Electronic;
import com.neostra.electronic.ElectronicCallback;

import java.text.DecimalFormat;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;

/**
 * @author ：Chenjk
 * @version 1.0
 * @createTime ：2023/12/25 19:42
 **/
public class ScaleActivity extends AppCompatActivity implements View.OnClickListener, ElectronicCallback {
    private TextView mTvWeight;
    private TextView mTvWeightSteady;
    private Button mBtnZero;
    private Button mBtnPeel;
    private String TAG = this.getClass().getSimpleName();
    private StringBuilder sb = new StringBuilder();
    private String mWeightState = null;
    private DecimalFormat df = new DecimalFormat("#######0.000");
    private Electronic mElectronic = null;
    private String devicePath = "/dev/ttyS4";

    private final static String SCALE_UNSTABLE = "55";
    private final static String SCALE_STABLE = "53";
    private final static String SCALE_OVER_WEIGHT = "46";

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_electronic_test);
        initView();
        initSerialPort();
    }

    private void initView() {
        mTvWeight = findViewById(R.id.tv_weight_value);
        mTvWeightSteady = findViewById(R.id.tv_weight_steady);
        mBtnZero = findViewById(R.id.btn_zero);
        mBtnPeel = findViewById(R.id.btn_peel);

        mBtnZero.setOnClickListener(this);
        mBtnPeel.setOnClickListener(this);
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.btn_peel:
                if (SCALE_UNSTABLE.equals(mWeightState)) {
                    Toast.makeText(ScaleActivity.this, getString(R.string.weighing_unstable_please_wait), Toast.LENGTH_SHORT).show();
                    return;
                }
                mElectronic.removePeel();
                break;
            case R.id.btn_zero:
                if (SCALE_UNSTABLE.equals(mWeightState)) {
                    Toast.makeText(ScaleActivity.this, getString(R.string.weighing_unstable_please_wait), Toast.LENGTH_SHORT).show();
                    return;
                }
                mElectronic.turnZero();
                break;
            default:
                break;
        }
    }


    private void initSerialPort() {
        // /dev/ttyS3 D3-510
        Log.d("electronic", " hhhhhh===> " + Electronic.getDevicePath());
        String devPath = getIntent().getStringExtra("devPath");
        if (TextUtils.isEmpty(devPath)) {
            devPath = Electronic.getDevicePath();
        }
        mElectronic = new Electronic.Builder()
                .setDevicePath(devPath)
                .setReceiveCallback(this).builder();
    }

    @Override
    public void electronicStatus(String weight, String weightStatus) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                if (SCALE_OVER_WEIGHT.equals(mWeightState)) {
                    //超重
                    mTvWeightSteady.setText(getString(R.string.over_weight));
                } else if (SCALE_STABLE.equals(mWeightState)) {
                    //稳定
                    mTvWeightSteady.setText(getString(R.string.steady));
                    mTvWeight.setTextColor(getColor(android.R.color.holo_green_light));
                } else if (SCALE_UNSTABLE.equals(mWeightState)) {
                    //未稳定
                    mTvWeightSteady.setText(getString(R.string.unsteady));
                    mTvWeight.setTextColor(getColor(android.R.color.holo_red_light));
                }
                mTvWeight.setText(weight);
            }

        });
        mWeightState = weightStatus;
    }
}
