package com.imin.apitest.biz.scan.ms1;

import android.content.Intent;
import android.media.Image;
import android.os.Build;
import android.os.Bundle;
import android.util.Log;
import android.util.Size;
import android.view.Display;
import android.widget.Button;
import android.widget.TextView;

import com.google.common.util.concurrent.ListenableFuture;
import com.google.mlkit.vision.barcode.Barcode;
import com.google.mlkit.vision.barcode.BarcodeScanner;
import com.google.mlkit.vision.barcode.BarcodeScanning;
import com.google.mlkit.vision.common.InputImage;
import com.imin.apitest.R;
import com.imin.apitest.biz.scan.Constant;
import com.imin.apitest.biz.scan.ResultActivity;
import com.imin.apitest.utils.DisplayUtils;
import com.imin.scan.ScanUtils;
import com.imin.zxing.BarcodeFormat;

import java.util.concurrent.ExecutionException;

import androidx.annotation.NonNull;
import androidx.annotation.OptIn;
import androidx.annotation.RequiresApi;
import androidx.appcompat.app.AppCompatActivity;
import androidx.camera.core.Camera;
import androidx.camera.core.CameraSelector;
import androidx.camera.core.ExperimentalGetImage;
import androidx.camera.core.ImageAnalysis;
import androidx.camera.core.ImageCapture;
import androidx.camera.core.ImageProxy;
import androidx.camera.core.Preview;
import androidx.camera.lifecycle.ProcessCameraProvider;
import androidx.camera.view.LifecycleCameraController;
import androidx.camera.view.PreviewView;
import androidx.core.content.ContextCompat;

/**
 * @author ：Chenjk
 * @version 1.0
 * @createTime ：2023/12/29 17:53
 **/
public class ScanTest extends AppCompatActivity {
    private static final String TAG = "ScanTest";
    private TextView tv_result;
    private Button btnPass, btnRetry, btnFail;
    PreviewView previewView;
    private ListenableFuture<ProcessCameraProvider> cameraProviderFuture;
    private BarcodeScanner barcodeScanner;
    private boolean isPause = false;
    ScanUtils scanUtils;
    private ProcessCameraProvider cameraProvider;
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.scan_test);
        //tv_result = (TextView)findViewById(R.id.tv_result);

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            setupCamera();
            scanUtils = ScanUtils.getInstance(this);
            scanUtils.initBeepSound(true,R.raw.beep);
        }
    }

    @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
    private void setupCamera() {
        previewView = findViewById(R.id.preview_view);
        cameraProviderFuture = ProcessCameraProvider.getInstance(this);

        cameraProviderFuture.addListener(() -> {
            try {
                cameraProvider = cameraProviderFuture.get();

                Preview preview = new Preview.Builder().build();
                preview.setSurfaceProvider(previewView.getSurfaceProvider());

                CameraSelector cameraSelector = new CameraSelector.Builder().requireLensFacing(CameraSelector.LENS_FACING_BACK).build();
                previewView.setController(new LifecycleCameraController(this));
                Camera camera = cameraProvider.bindToLifecycle(this, cameraSelector, preview);
                new ImageCapture.Builder().setCaptureMode(ImageCapture.CAPTURE_MODE_MAXIMIZE_QUALITY);
                // 创建BarcodeScanner实例
                barcodeScanner = BarcodeScanning.getClient();
                // 创建ImageAnalysis实例
                ImageAnalysis imageAnalysis = new ImageAnalysis.Builder()
                        .setBackpressureStrategy(ImageAnalysis.STRATEGY_KEEP_ONLY_LATEST)
                        .setTargetResolution(new Size(DisplayUtils.dp2px(this,300), DisplayUtils.dp2px(this,300)))
                        .build();

                // 设置ImageAnalysis分析器
                imageAnalysis.setAnalyzer(ContextCompat.getMainExecutor(this), new ImageAnalysis.Analyzer() {
                    @Override
                    @OptIn(markerClass = ExperimentalGetImage.class)
                    public void analyze(@NonNull ImageProxy image) {
                        Log.d("anylyze", "analyze--------->");
                        // 获取ImageProxy对象
                        Image mediaImage = image.getImage();
                        if (mediaImage != null) {
                            // 创建InputImage对象
                            InputImage inputImage = InputImage.fromMediaImage(mediaImage, image.getImageInfo().getRotationDegrees());

                            // 使用BarcodeScanner识别二维码
                            barcodeScanner.process(inputImage).addOnSuccessListener(barcodes -> {
                                for (Barcode barcode : barcodes) {
//                                            if (barcode.getFormat() == Barcode.FORMAT_QR_CODE) {
                                    String qrCodeValue = barcode.getRawValue();
                                    releaseCamera();
                                    // 在UI线程中更新TextView
                                    runOnUiThread(() -> {
                                        handlerResult(qrCodeValue);
                                    });
                                    return;
//                                            }
                                }
                            }).addOnFailureListener(e -> {
                                // 处理扫描失败的情况
                                Log.e("QrScanActivity", "Failed to scan barcode", e);
                            }).addOnCompleteListener(task -> image.close());
                        } else {
                            image.close();
                        }
                    }
                });

                // 绑定ImageAnalysis实例到相机生命周期中
                cameraProvider.bindToLifecycle(this, cameraSelector, imageAnalysis);
            } catch (ExecutionException | InterruptedException e) {
                Log.e("QrScanActivity", "Failed to initialize camera", e);
            }
        }, ContextCompat.getMainExecutor(this));
    }

    private void handlerResult(String qrCodeValue) {
            scanUtils.playBeepSoundAndVibrate();

            Intent resultIntent = new Intent(ScanTest.this, ResultActivity.class);
            Bundle bundle = new Bundle();
            bundle.putString(Constant.INTENT_EXTRA_KEY_QR_SCAN, qrCodeValue);
            bundle.putString(Constant.INTENT_EXTRA_KEY_QR_TYPE, "");

            resultIntent.putExtras(bundle);
            startActivity(resultIntent);
            finish();
    }

    /**
     * 释放相机
     */
    protected void releaseCamera() {
        if (barcodeScanner != null) {
            barcodeScanner.close();
            try {
                cameraProvider.unbindAll();
            } catch (Exception e) {
                e.printStackTrace();
            }
            cameraProviderFuture.cancel(false);
        }
    }

    @Override
    protected void onResume() {
        // TODO Auto-generated method stub
        super.onResume();
        isPause = false;
    }

    @Override
    protected void onDestroy() {
        if (barcodeScanner != null) {
            barcodeScanner.close();
        }
        super.onDestroy();
    }

    @Override
    protected void onPause() {
        super.onPause();
        isPause = true;
    }
}
